//+------------------------------------------------------------------+
//|                                                                  |
//|             Puntatore Segna Minimi - Solospread                  |
//|             Visualizza SLIP e R2S                                |
//|                                                                  |
//+------------------------------------------------------------------+
#property strict
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Aqua
#property indicator_width1 2
#property indicator_color2 Blue
#property indicator_width2 1
#property indicator_color3 Red
#property indicator_width3 2
#property indicator_color4 Chocolate
#property indicator_width4 1

//---- input parameters
extern int Slip_Period = 15; //Periodo cerca Min_Slip
extern double Fctr1 = 100; //Slip factor
extern int R2S_Period = 15; //Periodo cerca Max_R2S
extern double Fctr2 = 100; //R2S factor
extern int HL_Period = 30; //Periodo cerca Max e Min
extern int MA_Period = 3; //Periodo smooting SMA
extern int RSI_Period = 14;  //Periodo RSI
extern int Band_Period = 20;  //Periodo Upper Bollinger  
extern int Band_Dev = 4;  //Dev-Std Bollinger

//---- buffers
double Slip[1999], R2S[1999], M_Slip[1999], M_R2S[1999];
double Punta_Min[1999], Punta_Max[1999];
double RA[1999], RB[1999], RC[1999];
double RAA[1999], RCC[1999];
double mio_RSI, mio_UBand, mio_Min, mio_Max, mio_MA, mio_Min2, mio_Max2, mio_MA2;
double sum, Thank, Please, QQ, KK;
int i, j, limit;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   IndicatorBuffers(4);

   SetIndexBuffer(0, Slip);
   SetIndexStyle(0, DRAW_LINE);
   SetIndexBuffer(1, M_Slip);
   SetIndexStyle(1, DRAW_LINE);
   SetIndexBuffer(2, R2S);
   SetIndexStyle(2, DRAW_LINE);
   SetIndexBuffer(3, M_R2S);
   SetIndexStyle(3, DRAW_LINE);

   //SetIndexDrawBegin(0, 0);
   IndicatorShortName("Slip / Min_Slip / R2S / Max_R2S");
   SetIndexLabel(0, "Slip");
   SetIndexLabel(1, "Min_Slip");
   SetIndexLabel(2, "R2S");
   SetIndexLabel(3, "Max_R2S");

   ArraySetAsSeries(RA,true);
   ArraySetAsSeries(RB,true);
   ArraySetAsSeries(RC,true);

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars = IndicatorCounted();
   if(counted_bars < 0) 
       return(-1);
   if(counted_bars > 0) 
       counted_bars--;
   limit = Bars - counted_bars;
   if(limit > 1999) limit = 1999-1;
   
   // calcola Punta_Massimi     
   for(i = 0; i < limit; i++)
   {
      mio_Max2 = High[iHighest(NULL,0,MODE_HIGH,HL_Period,i)];
      mio_Min2 = Low[iLowest(NULL,0,MODE_LOW,HL_Period,i)];
      mio_MA2 = iMA(NULL, 0, MA_Period, 0, MODE_SMA, PRICE_CLOSE, i);

      mio_Max = iRSI(NULL,0,RSI_Period,PRICE_CLOSE,i);
      mio_Min = mio_Max;
      for(j = i+1; j<(i+HL_Period); j++)
      {
         mio_RSI = iRSI(NULL,0,RSI_Period,PRICE_CLOSE,j);
         if(mio_RSI > mio_Max) mio_Max = mio_RSI;
         if(mio_RSI < mio_Min) mio_Min = mio_RSI;
      }

      sum = iRSI(NULL,0,RSI_Period,PRICE_CLOSE,i);
      for(j = i+1; j<(i+MA_Period); j++) sum += iRSI(NULL,0,RSI_Period,PRICE_CLOSE,j);
      mio_MA = sum / MA_Period;
      
      //Bande solospread 
      RA[i] = ((mio_Max - Close[i]) + (mio_MA  * mio_Max)) / High[i];
      RC[i] = (mio_Max - mio_MA) + (Close[i] * mio_Max) / Close[i];

      //Bande solospread modificate 
      mio_RSI = iRSI(NULL,0,RSI_Period,PRICE_CLOSE,i);
      RAA[i] = ((mio_Max - mio_RSI) + (mio_MA  * mio_Max2)) / High[i];
      RCC[i] = (mio_Max - mio_MA) + (mio_RSI * mio_Max2) / Close[i];
      double div = RC[i] + RCC[i];
      Punta_Max[i] = RA[i] + RAA[i];
      if(div != 0) Punta_Max[i] = (RA[i] + RAA[i]) / (RC[i] + RCC[i]);
   } 

   // Calcola Punta Minimi  
   for(i = 0; i < limit; i++)
   {
      mio_Max = High[iHighest(NULL,0,MODE_HIGH,HL_Period,i)];
      mio_Min = Low[iLowest(NULL,0,MODE_LOW,HL_Period,i)];
      mio_MA = iMA(NULL, 0, MA_Period, 0, MODE_SMA, PRICE_CLOSE, i);
      RA[i] = ((mio_Max - Close[i]) + (mio_MA  * mio_Max)) / High[i];
      RB[i] = ((mio_Min - Close[i]) + (mio_MA  * mio_Min)) / Low[i];
      RC[i] = (mio_Max - mio_MA) + (Close[i] * mio_Max) / Close[i];   
   }
      
   for(i = 0; i < limit; i++)
   {
      mio_RSI = iRSI(NULL,0,RSI_Period,PRICE_CLOSE,i);
      mio_UBand = iBands(NULL,0,Band_Period,Band_Dev,0,PRICE_CLOSE,MODE_UPPER,i); 
      Thank = iMAOnArray(RA, 0, 3, 0, MODE_SMA, i);
      Please = iMAOnArray(RB, 0, 3, 0, MODE_SMA, i);
      QQ = 0; 
      if(Please != 0) QQ = (RC[i] - RA[i]) / Please;
      KK = 0;
      if(Thank != 0)  KK = (mio_UBand - RB[i]) / Thank;
      Punta_Min[i] = 100 /(mio_RSI + QQ - KK) + 48;       
   }
   
   // Calcola Slip & R2S
   if(limit > (1999 - Slip_Period - R2S_Period)) limit = 1999 - Slip_Period - R2S_Period - 1; 
   for(i = 0; i < (limit); i++)
   {
      Slip[i] = -1999;
      if(Punta_Min[i] != 0) Slip[i] = Fctr1 * Punta_Max[i] / Punta_Min[i];
      M_Slip[i] = Slip[i];
      for(j = i+1; j<(i+Slip_Period); j++)
      {
         Slip[j] = -1999;
         if(Punta_Min[j] != 0) Slip[j] = Fctr1 * Punta_Max[j] / Punta_Min[j];
         if(Slip[j] < M_Slip[i]) M_Slip[i] = Slip[j];
      }
      
      R2S[i] = -1999;
      if(Punta_Max[i] != 0) R2S[i] = (Punta_Min[i] / Punta_Max[i]) / Fctr2;
      M_R2S[i] = R2S[i];
      for(j = i+1; j<(i+R2S_Period); j++)
      {
         R2S[j] = -1999;
         if(Punta_Max[j] != 0) R2S[j] = (Punta_Min[j] / Punta_Max[j]) / Fctr2;
         if(R2S[j] > M_R2S[i]) M_R2S[i] = R2S[j];
      }
   }

   return(0);
  }
//+------------------------------------------------------------------+